@@ -30,8 +30,14 @@ duckdbfs_env <- new.env()
 #' @inheritParams duckdb::duckdb
 #' @param autoload_exts should we auto-load extensions?  TRUE by default,
 #' can be configured with `options(duckdbfs_autoload_extensions = FALSE)`
+#' @param with_spatial install (if missing) and load spatial extension, default TRUE.
+#'  Opt out by closing any active cached connection first (with
+#'  `close_connection()`) and re-instantiating the with
+#'  `connect(with_spatial = FALSE)`.
+#' @param with_h3 install (if missing) and load  the h3 spatial index extension.
 #' @returns a [duckdb::duckdb()] connection object
-#' @examples
+#' @aliases cached_connection duckdb_connect
+#' @examplesIf interactive()
 #'
 #' con <- cached_connection()
 #' close_connection(con)
@@ -44,7 +50,11 @@ cached_connection <- function(dbdir = ":memory:",
                               config = list(temp_directory = tempfile()),
                               autoload_exts =
                                 getOption("duckdbfs_autoload_extensions",
-                                          TRUE)
+                                          TRUE),
+                              with_spatial = getOption("duckdbfs_autoload_extensions",
+                                                       TRUE),
+                              with_h3 = not_windows() && getOption("duckdbfs_autoload_extensions",
+                                                                   TRUE)
                               ) {
 
   #conn <- mget("duckdbfs_conn", envir = duckdbfs_env,
@@ -71,14 +81,25 @@ cached_connection <- function(dbdir = ":memory:",
                            bigint = bigint,
                            config = config)
 
+    if (with_spatial) {
+      # can't use load_spatial here, creates infinite recursion
+      DBI::dbExecute(conn, "INSTALL spatial;")
+      DBI::dbExecute(conn, "LOAD spatial;")
+    }
 
-    options(duckdbfs_conn = conn)
-    # assign("duckdbfs_conn", conn, envir = duckdbfs_env)
+    if (with_h3) {
+      DBI::dbExecute(conn, "INSTALL h3 from community;")
+      DBI::dbExecute(conn, "LOAD h3;")
+    }
 
     if (autoload_exts) {
       DBI::dbExecute(conn, "SET autoinstall_known_extensions=1;")
       DBI::dbExecute(conn, "SET autoload_known_extensions=1;")
     }
+
+    options(duckdbfs_conn = conn)
+    # assign("duckdbfs_conn", conn, envir = duckdbfs_env)
+
   }
 
   ## create finalizer to avoid duckdb complaining that connection
@@ -89,6 +110,8 @@ cached_connection <- function(dbdir = ":memory:",
   conn
 }
 
+
+
 #' close connection
 #'
 #' @param conn a duckdb connection (leave blank)
@@ -98,7 +121,7 @@ cached_connection <- function(dbdir = ":memory:",
 #' Then clear cached reference to avoid using a stale connection
 #' This avoids complaint about connection being garbage collected.
 #' @returns returns nothing.
-#' @examples
+#' @examplesIf interactive()
 #'
 #' close_connection()
 #'
@@ -117,3 +140,13 @@ close_connection <- function(conn = cached_connection()) {
   rm(conn)
 }
 
+
+
+#' @export
+duckdb_connect <- cached_connection
+
+
+
+not_windows <- function() {
+  tolower(Sys.info()[["sysname"]]) != "windows"
+}
